package com.ablecloud.weixin.service;

import com.ablecloud.bridge.wechat.ACBridgeWeChat;
import com.ablecloud.bridge.wechat.ACWeChatConfig;
import com.ablecloud.common.*;
import com.ablecloud.service.AC;
import com.ablecloud.weixin.common.ACWXMsgType;
import com.ablecloud.weixin.common.ACWXXmlResp;
import com.ablecloud.weixin.utils.AccessTokenKeeper;
import com.ablecloud.weixin.utils.JsApiTicketKeeper;
import com.ablecloud.weixin.utils.WXMPApi;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 微信公众号推送的事件的处理工具。
 */
public class ACWXMessageService {
    private static final Logger logger = LoggerFactory.getLogger(ACWXMessageService.class);
    private AC ac;
    private ACWeChatConfig bridgeConfig;

    public ACWXMessageService(AC ac) {
        this.ac = ac;
    }

    public String handleMsg(HttpServletRequest req, HttpServletResponse resp) throws Exception {
        // parse xml data from req
        Map<String, String> reqMap = parseXml(req.getInputStream());
        logger.info("req data = " + reqMap);
        String msgType = reqMap.get("MsgType");
        String fromUser = reqMap.get("FromUserName");
        String toUser = reqMap.get("ToUserName");

        ACBridgeWeChat bridge = new ACBridgeWeChat(AccessTokenKeeper.getTokenStr(), JsApiTicketKeeper.getTicketStr(), this.ac);

        switch (msgType) {
            case ACWXMsgType.EVENT: {
                String event = reqMap.get("Event");
                switch (event) {
                    case ACWXMsgType.Event.SUBSCRIBE:
                        String unionId = WXMPApi.getUserUnionId("fromUser");
                        bridge.onEventSubscribe(fromUser, unionId);
                        String xmlResp = "欢迎关注AbleCloud智能硬件公众号，开启全新的微信硬件功能开发模式。<a href='http://docs.ablecloud.cn/'>了解更多</a>";
                        return ACWXXmlResp.buildTextResp(fromUser, toUser, xmlResp);
                    case ACWXMsgType.Event.UNSUBSCRIBE:
                        try {
                            bridge.onEventUnsubscribe(fromUser, toUser);
                        } catch (Exception e) {
                            logger.warn("Error in handling event of " + ACWXMsgType.Event.UNSUBSCRIBE + ": " + e.getMessage());
                        }
                        break;
                    default:
                        break;
                }
                break;
            }
            case ACWXMsgType.DEVICE_EVENT: {
                String event = reqMap.get("Event");
                switch (event) {
                    case ACWXMsgType.DeviceEvent.BIND: {
                        try {
                            String deviceId   = reqMap.get("DeviceID");
                            String content    = reqMap.get("Content");
                            String subDomain  = "";     // 假设content中含有二维码信息。
                            String deviceName = ACWXServer.config.getDefaultDeviceName();
                            if (deviceName == null || deviceName.isEmpty())
                                deviceName = deviceId;
                            bridge.onDeviceEventBind(fromUser, deviceId, content, deviceName, subDomain);
                        } catch (Exception e) {
                            logger.warn("Error in handling event of " + ACWXMsgType.DEVICE_EVENT + ": " + e.getMessage());
                        }
                        break;
                    }
                    case ACWXMsgType.DeviceEvent.UNBIND: {
                        try {
                            String deviceId   = reqMap.get("DeviceID");
                            String deviceType = reqMap.get("DeviceType");
                            String subDomain  = "light";    // 开发者需要自己找到该设备所属的子域的名字。
                            bridge.onDeviceEventUnbind(fromUser, deviceId, deviceType, subDomain);
                        } catch (Exception e) {
                            logger.warn("Error in handling event of " + ACWXMsgType.DeviceEvent.UNBIND + ": " + e.getMessage());
                        }
                        break;
                    }
                    case ACWXMsgType.DeviceEvent.SUBSCRIBE_STATUS: {
                        String deviceId   = reqMap.get("DeviceID");
                        String deviceType = reqMap.get("DeviceType");
                        return bridge.onDeviceEventSubscribeStatus(fromUser, toUser, deviceId, deviceType, false);
                    }
                    case ACWXMsgType.DeviceEvent.UNSUBSCRIBE_STATUS: {
                        String deviceId   = reqMap.get("DeviceID");
                        String deviceType = reqMap.get("DeviceType");
                        return bridge.onDeviceEventUnsubscribeStatus(fromUser, toUser, deviceId, deviceType, false);
                    }
                }
            }
        }
        return "";
    }

    protected static Map<String, String> parseXml(InputStream in) throws DocumentException, IOException {
        Map<String, String> map = new HashMap<String, String>();
        SAXReader reader = new SAXReader();
        Document document = reader.read(in);
        Element root = document.getRootElement();
        List<Element> elementList = root.elements();
        for (Element element : elementList) {
            map.put(element.getName(), element.getText());
        }
        return map;
    }
}
