package com.ablecloud.weixin.service;

import com.ablecloud.service.AC;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 处理微信公众号平台推送的事件及消息。
 */
public class ACWXMessageServlet extends ACWXServlet {
    private static final long serialVersionUID = 1L;
    private ACWXMessageService acwxMessageService;

    public ACWXMessageServlet(AC ac) {
        acwxMessageService = new ACWXMessageService(ac);
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        // verify developer's signature
        try {
            String timestamp = req.getParameter(TIMESTAMP);
            String nonce = req.getParameter(NONCE);
            String signature = req.getParameter(SIGNATURE);
            String echostr = req.getParameter(ECHOSTR);

            if (verifySignature(timestamp, nonce, signature)) {
                writeXmlResp(echostr, resp);
            } else {
                writeXmlResp("invalid signature", resp);
            }
        } catch (Exception e) {
            e.printStackTrace();
            writeXmlResp("", resp);
        }
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        try {
            req.setCharacterEncoding("UTF-8");
            resp.setCharacterEncoding("UTF-8");

            // verify signature
            String timestamp = req.getParameter(TIMESTAMP);
            String nonce = req.getParameter(NONCE);
            String signature = req.getParameter(SIGNATURE);
            if (!verifySignature(timestamp, nonce, signature)) {
                writeXmlResp("invalid signature", resp);
                return;
            }

            String xmlResp = acwxMessageService.handleMsg(req, resp);
            xmlResp = xmlResp == null ? "" : xmlResp;
            writeXmlResp(xmlResp, resp);
        } catch (Exception e) {
            e.printStackTrace();
            writeXmlResp("", resp);
        }
    }
}
