package com.ablecloud.weixin.service;

import com.ablecloud.common.ACConfig;
import com.ablecloud.service.AC;
import com.ablecloud.weixin.utils.AccessTokenKeeper;
import com.ablecloud.weixin.utils.JsApiTicketKeeper;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import com.ablecloud.bridge.wechat.ACBridgeWeChat;
import com.ablecloud.bridge.wechat.ACWeChatConfig;

/**
 * 获取设备的二维码。
 */
public class ACWXQrCodeServlet extends ACWXServlet {
	private static final Logger logger = LoggerFactory.getLogger(ACWXQrCodeServlet.class);
	private AC ac;

	public ACWXQrCodeServlet(AC ac) {
		this.ac = ac;
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {
        ServletOutputStream out = resp.getOutputStream();
        try {
        	String physicalDeviceId = req.getParameter("physicalDeviceId");
			String subDomain		= req.getParameter("subDomain");
            String content = req.getParameter("content");   // content: basic (WeiXin), extended (with AbleCloud share-code)
            if (content == null)
                content = "extended";
			ACBridgeWeChat bridge = new ACBridgeWeChat(AccessTokenKeeper.getTokenStr(), JsApiTicketKeeper.getTicketStr(), this.ac);
			String qrCode;
            if (content.compareToIgnoreCase("extended") == 0) {
                int uid = Integer.parseInt(req.getParameter("uid"));
                int timeout = Integer.parseInt(req.getParameter("timeout"));
				qrCode  = bridge.getDeviceQRCode(physicalDeviceId, subDomain, uid, timeout);
            } else {
				qrCode  = bridge.getDeviceQRCode(physicalDeviceId, subDomain);
			}
			resp.setContentType("application/json");
			resp.setCharacterEncoding("UTF-8");
			JSONObject json = new JSONObject();
			json.put("qrcode", qrCode);
			out.print(json.toString());
        	out.flush();
        } catch (Exception e) {
            e.printStackTrace();
            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
        } finally {
        	if (out != null) {
        		try {
        			out.close();
        		} catch (Exception e) {
        			e.printStackTrace();
        		}
        	}
        }
    }
}