package com.ablecloud.weixin.service;

import com.ablecloud.cloudservice.ACCloud;
import com.ablecloud.service.AC;
import com.ablecloud.weixin.common.ACWXConfig;
import org.apache.commons.configuration.ConfigurationException;
import org.eclipse.jetty.server.NCSARequestLog;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.HandlerCollection;
import org.eclipse.jetty.server.handler.RequestLogHandler;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.webapp.WebAppContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;

/**
 * Main函数。
 */
public class ACWXServer {
    private static final Logger logger = LoggerFactory.getLogger(ACWXServer.class);

    public static ACWXConfig config;

    public static void main(String[] args) throws Exception {
        try {
            config = new ACWXConfig("./config/wc.conf");

            AC ac = new ACCloud(config);

            WebAppContext webAppContext = new WebAppContext();
            webAppContext.setContextPath("/");

            ServletContextHandler servletContextHandler = new ServletContextHandler(ServletContextHandler.SESSIONS);
            servletContextHandler.setContextPath("/weixin");
            servletContextHandler.addServlet(new ServletHolder(new ACWXMessageServlet(ac)), "/message");
            servletContextHandler.addServlet(new ServletHolder(new ACWXAirKissServlet()), "/airkiss");
            servletContextHandler.addServlet(new ServletHolder(new ACWXQrCodeServlet(ac)), "/qrcode");

            RequestLogHandler requestLogHandler = new RequestLogHandler();
            NCSARequestLog ncsaRequestLog = new NCSARequestLog("./log/http.log");
            ncsaRequestLog.setRetainDays(90);
            ncsaRequestLog.setAppend(true);
            ncsaRequestLog.setExtended(false);
            ncsaRequestLog.setLogTimeZone("GMT");
            requestLogHandler.setRequestLog(ncsaRequestLog);

            HandlerCollection handlers = new HandlerCollection();
            handlers.addHandler(servletContextHandler);
            handlers.addHandler(webAppContext);
            handlers.addHandler(requestLogHandler);

            Server server = new Server(new InetSocketAddress(config.getHost(), config.getPort()));
            server.setHandler(handlers);
            server.start();
            server.join();
        } catch (ConfigurationException e) {
            logger.error("load config file error: ", e);
        }
    }
}
