package com.ablecloud.weixin.service;

import com.ablecloud.weixin.common.ACWXSigner;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.Writer;

/**
 * Servlet的基类。
 */
@WebServlet("/")
public class ACWXServlet extends HttpServlet {
    private static final Logger logger = LoggerFactory.getLogger(ACWXServlet.class);
    protected static final String TIMESTAMP = "timestamp";
    protected static final String NONCE = "nonce";
    protected static final String SIGNATURE = "signature";
    protected static final String ECHOSTR = "echostr";

    public ACWXServlet() {}

    protected void writeXmlResp(String data, HttpServletResponse resp) {
        writeResponse(data, "text/xml", resp);
    }

    protected void writeHtmlResp(String data, HttpServletResponse resp) {
        writeResponse(data, "text/html", resp);
    }

    protected void writeJsonResp(String data, HttpServletResponse resp) {
        writeResponse(data, "text/json", resp);
    }

    protected static boolean verifySignature(String timestamp, String nonce, String signature) {
        try {
            String actualSignature = ACWXSigner.genSignature(ACWXServer.config.getToken(), timestamp, nonce);
            if (signature.equals(actualSignature)) {
                return true;
            } else {
                logger.warn("verify signature failed, expected[" + signature +
                            "] actual[" + actualSignature + "]");
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    protected static String getRootURL(HttpServletRequest req) {
        String url = req.getScheme() + "://" + req.getServerName();
        if (req.getServerPort() != 80) {
            url += ":" + req.getServerPort();
        }
        return url;
    }

    protected static String getOriginalRequestURL(HttpServletRequest req, boolean withQueryString) {
        String url = getRootURL(req) + req.getContextPath() + req.getServletPath();
        String pathInfo = req.getPathInfo();
        if (pathInfo != null) {
            url += pathInfo;
        }
        if (withQueryString) {
            String q = req.getQueryString();
            if (q != null)
                url += "?" + q;
        }
        return url;
    }

    private void writeResponse(String data, String contentType, HttpServletResponse resp) {
        Writer out = null;
        try {
            resp.setContentType(contentType + ";charset=UTF-8");
            resp.setStatus(HttpServletResponse.SC_OK);
            out = resp.getWriter();
            out.append(data);
            out.flush();
        } catch (IOException e) {
            logger.warn("write response error ", e.getMessage());
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    // ignore
                }
            }
        }
    }
}
