package com.ablecloud.weixin.utils;

import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by chenpeng on 15-3-17.
 */
public class AccessTokenKeeper {
    private static final Logger logger = LoggerFactory.getLogger(AccessTokenKeeper.class);
    private static AccessToken accessToken;

    private static ScheduledExecutorService timer = Executors
            .newSingleThreadScheduledExecutor(new ThreadFactory() {
                @Override
                public Thread newThread(Runnable r) {
                    Thread t = new Thread(r);
                    t.setDaemon(true);
                    return t;
                }
            });

    static {
        init();
    }

    public static String getTokenStr() {
        return getAccessToken().getAccess_token();
    }

    public static AccessToken getAccessToken() {
        return accessToken;
    }

    // refresh token per 10 seconds
    public static synchronized String refreshAndGetToken() {
        AccessToken accessToken = getAccessToken();
        if (accessToken == null ||
                System.currentTimeMillis() - accessToken.getCreateTime() > 10000) {
            refreshToken();
        }
        return getTokenStr();
    }

    private static void refreshToken() {
        try {
            AccessToken token = WXMPApi.getAccessToken();
            saveAccessToken(token);
        } catch (Exception e) {
            logger.warn("save access token error ", e);
        }
    }

    // TODO, you'd better save token in disk or other storage system
    private static void saveAccessToken(AccessToken token) {
        accessToken = token;
    }

    public static void init() {
        if (getAccessToken() == null) {
            refreshToken();
        }
        initTimer(getAccessToken());
    }

    private static void initTimer(AccessToken accessToken) {
        long refreshTime = accessToken.getExpires_in() * 2 / 3;
        long delay = (long)(100 * (new Random().nextDouble()));
        timer.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                logger.info("refresh token thread start");
                AccessToken token = getAccessToken();
                if (token == null ||
                        System.currentTimeMillis() - token.getCreateTime() > 200000) {
                    refreshToken();
                }
            }
        }, delay, refreshTime, TimeUnit.SECONDS);

        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
            @Override
            public void run() {
                timer.shutdown();
            }
        }));
    }
}
