package com.ablecloud.weixin.utils;

import net.sf.json.JSONObject;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpResponseException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.util.EntityUtils;
import org.apache.http.client.ResponseHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.entity.StringEntity;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

//import javax.net.ssl.SSLContext;
//import javax.net.ssl.TrustManager;
//import javax.net.ssl.X509TrustManager;
//import org.apache.http.conn.ClientConnectionManager;
//import org.apache.http.conn.scheme.Scheme;
//import org.apache.http.conn.scheme.SchemeRegistry;
//import org.apache.http.conn.ssl.SSLSocketFactory;
//import java.security.GeneralSecurityException;
//import java.security.cert.CertificateException;
//import java.security.cert.X509Certificate;

/**
 * Created by chenpeng on 15-3-16.
 */
public class HttpUtil {
    private static final Logger logger = LoggerFactory.getLogger(HttpUtil.class);

    public static String doGet(String url) {
        String realUrl = url.replace("ACCESS_TOKEN", AccessTokenKeeper.getTokenStr());
        String result = executeGet(realUrl);
        JSONObject json = JSONObject.fromObject(result);
        // retry in case of token expired
        if (json.containsKey("errcode") &&
                (json.getInt("errcode") == 40001
                || json.getInt("errcode") == 40014
                || json.getInt("errcode") == 41001
                || json.getInt("errcode") == 42001)) {
            realUrl = url.replace("ACCESS_TOKEN",
                    AccessTokenKeeper.refreshAndGetToken());
            result = executeGet(realUrl);
        }
        return result;
    }

    public static String executeGet(String url) {
        HttpGet httpGet = new HttpGet(url);
        try {
            HttpClient httpClient = new DefaultHttpClient();
            logger.debug("url:" + url);
            HttpResponse response = httpClient.execute(httpGet);
            logger.debug("resp:" + response.toString());
            String resultContent = new Utf8ResponseHandler()
                                        .handleResponse(response);
            return resultContent;
        } catch (Exception e) {
            logger.warn("execute http get error ", e);
            throw new RuntimeException(e);
        } finally {
            httpGet.releaseConnection();
        }
    }

    public static String doPost(String url, String body) {
        String realUrl = url.replace("ACCESS_TOKEN", AccessTokenKeeper.getTokenStr());
        String result = executePost(realUrl, body);
        JSONObject json = JSONObject.fromObject(result);
        // retry in case of token expired
        if (json.containsKey("errcode") &&
                (json.getInt("errcode") == 40001
                        || json.getInt("errcode") == 40014
                        || json.getInt("errcode") == 41001
                        || json.getInt("errcode") == 42001)) {
            realUrl = url.replace("ACCESS_TOKEN",
                    AccessTokenKeeper.refreshAndGetToken());
            result = executePost(realUrl, body);
        }
        return result;
    }

    public static String executePost(String url, String body) {
        HttpPost httpPost = new HttpPost(url);
        try {
            HttpClient httpClient = new DefaultHttpClient();
            StringEntity entity = new StringEntity(body, "UTF-8");
            httpPost.setEntity(entity);
            HttpResponse response = httpClient.execute(httpPost);
            String resultContent = new Utf8ResponseHandler()
                                        .handleResponse(response);
            return resultContent;
        } catch (Exception e) {
            logger.warn("execute http post error ", e);
            throw new RuntimeException(e);
        } finally {
            httpPost.releaseConnection();
        }
    }

    static class Utf8ResponseHandler implements ResponseHandler<String> {
        public String handleResponse(final HttpResponse resp)
            throws HttpResponseException, IOException {
            final StatusLine statusLine = resp.getStatusLine();
            final HttpEntity entity = resp.getEntity();
            if (statusLine.getStatusCode() >= 300) {
                EntityUtils.consume(entity);
                throw new HttpResponseException(statusLine.getStatusCode(),
                        statusLine.getReasonPhrase());
            }
            return entity == null ? null : EntityUtils.toString(entity, "UTF-8");
        }
    }
    /*
    private static HttpClient createSSLInsecureClient() throws GeneralSecurityException {
        try {
            HttpClient httpClient = new DefaultHttpClient();
            SSLContext sslContext = SSLContext.getInstance("SSL");
            X509TrustManager trustManager = new X509TrustManager() {
                @Override
                public void checkClientTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {

                }

                @Override
                public void checkServerTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {

                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
            };
            sslContext.init(null, new TrustManager[]{trustManager}, null);
            SSLSocketFactory sslSocketFactory = new SSLSocketFactory(sslContext);
            ClientConnectionManager clientConnectionManager = httpClient.getConnectionManager();
            SchemeRegistry schemeRegistry = clientConnectionManager.getSchemeRegistry();
            schemeRegistry.register(new Scheme("https", 443, sslSocketFactory));
            return httpClient;
        } catch (Exception e) {
            logger.error("create http client support https error ", e);
            return null;
        }
    }
    */

    public static void addCookies(HttpServletResponse resp, String openId, String majorDomain, String subDomain) {
        Cookie cookie;
        cookie = new Cookie("wxOpenId", openId);
        cookie.setPath("/");
        resp.addCookie(cookie);
        cookie = new Cookie("acMajorDomain", majorDomain);
        cookie.setPath("/");
        resp.addCookie(cookie);
        cookie = new Cookie("acSubDomain", subDomain);
        cookie.setPath("/");
        resp.addCookie(cookie);
        return;
    }
}
