package com.ablecloud.weixin.utils;

import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JsApiTicketKeeper {
    private static final Logger logger = LoggerFactory.getLogger(JsApiTicketKeeper.class);
    private static JsApiTicket ticket;

    private static ScheduledExecutorService timer = Executors
            .newSingleThreadScheduledExecutor(new ThreadFactory() {
                @Override
                public Thread newThread(Runnable r) {
                    Thread t = new Thread(r);
                    t.setDaemon(true);
                    return t;
                }
            });

    static {
        init();
    }

    public static String getTicketStr() {
        return getTicket().getTicket();
    }

    public static JsApiTicket getTicket() {
        if (ticket == null) {
            refreshTicket();
        }
        return ticket;
    }

    public static synchronized String refreshAndGetTicket() {
        JsApiTicket t = getTicket();
        if (t == null || t.hasExpired()) {
            refreshTicket();
        }
        return getTicketStr();
    }

    private static void refreshTicket() {
        try {
            JsApiTicket t = WXMPApi.getJsApiTicket();
            saveJsApiTicket(t);
        } catch (Exception e) {
            logger.warn("save JS-Api ticket error ", e);
        }
    }

    // TODO, you'd better save token in disk or other storage system
    private static void saveJsApiTicket(JsApiTicket t) {
        ticket = t;
    }

    public static void init() {
        if (getTicket() == null) {
            refreshTicket();
        }
        initTimer(getTicket());
    }

    private static void initTimer(JsApiTicket ticket) {
        long refreshTime = ticket.getExpires_in() * 2 / 3;
        long delay = (long)(100 * (new Random().nextDouble()));
        timer.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                logger.info("refresh JS-Api ticket thread start");
                JsApiTicket t = getTicket();
                if (t == null || t.hasExpired()) {
                    refreshTicket();
                }
            }
        }, delay, refreshTime, TimeUnit.SECONDS);

        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
            @Override
            public void run() {
                timer.shutdown();
            }
        }));
    }
}
