#!/bin/bash

lib_path="./lib"
framework_prefix="ac-weixin-server"
framework=""

conf_file=""
mode="production"
user_lib_path=""

usage() {
    echo "Usage: $0 [-c conf_file] [-l user_lib_path] [-m mode]"
    echo "    -c: specify user-defined configruration file."
    echo "    -l: specify library path which you will used excluding framework and framework's lib."
    echo "    -m: specify run mode. should be one of [test,production], default is production."
}

while getopts "c:l:m:h" opt
do
    case ${opt} in
        c)
            conf_file=$OPTARG
            ;;
        l)
            user_lib_path=$OPTARG
            ;;
        m)
            mode=$OPTARG
            if [ "${mode}" != "test" ] && [ "${mode}" != "production" ]
            then
                usage
                exit 1
            fi
            ;;
        h)
            usage
            exit 1
            ;;
        ?)
            usage
            exit 1
            ;;
    esac
done

framework=`ls ${lib_path}/${framework_prefix}*.jar`
fw_num=`echo ${framework} | awk -F' ' '{print NF}'`
if [ ${fw_num} -lt 1 ]
then
    echo "could not find cloud service framework jar file"
    exit 2
elif [ ${fw_num} -gt 1 ]
then
    echo "find more than one cloud service framework jar files["${framework}"]"
    exit 2
else
    framework=`echo ${framework} | awk -F' ' '{print $1}'`
fi

param="-Xbootclasspath/a:"
libs=`ls ${lib_path}/*.jar`
for lib in ${libs}
do
    param=${param}${lib}:
done

if [ -n "${user_lib_path}" ] && [ "${user_lib_path}" != "${lib_path}" ]
then
    libs=`ls ${user_lib_path}/*.jar`
    for lib in ${libs}
    do
        param=${param}${user_lib_path}"/"${lib}:
    done
fi

args="-Dmode=${mode} "
if [ -n "$conf_file" ]
then
    args=${args}"-Dconf_file=${conf_file}"
fi

exec java ${param} ${args} -Dfile.encoding=UTF-8 -Dsun.jnu.encoding=UTF-8 -Xmx2048m -Xms256m -jar ${framework}
